% implements newton's algorithm for root finding.

% p - a row vector for the coefficients of a polynomial. i.e. [1 2 3 4] is
% x^3 + 2*x^2 + 3*x^1 + 4
% x - complex number,  is a starting guess which should be close to the final root
% N - integer, is the number of times to run the iteration
% m - constant multiple in relaxed newton iterations
% tol - stop iterating when the difference in iteration becomes this
% r - matrix that is the result of the N iterations 
% s - paired matrix with each # of iterations until under desired tolerance
function [r,s] = newtp(p, x, N, m, tol, mr, critical,dotime)
    % get the derivative and second derivative if necessary
    dp = polyder(p);
    if mr dp2 = polyder(dp); end
    
    % set up a logical matrix to keep track of which numbers have converged
    % and keep track of previous iterations
    notconverged = true(size(x,1),size(x,2));
    prev = x;
    
    % set up matrix to keep track of when each entry converges
    conv = zeros(size(x,1),size(x,2));
    
    for j = 1:N
        if dotime
            tic;
        end
        % perform the iterations on only the values that have not converged
        % to within the desired tolerance
        
        if j <= critical
            if mr
                x = x - (polyval(p,x).*polyval(dp,x))./(polyval(dp,x).^2 - polyval(p,x).*polyval(dp2,x));
            else
                x = x - m*polyval(p,x)./polyval(dp,x);
            end
        else
            if mr
                x(notconverged) = x(notconverged) - (polyval(p,x(notconverged)).*polyval(dp,x(notconverged)))./(polyval(dp,x(notconverged)).^2 - polyval(p,x(notconverged)).*polyval(dp2,x(notconverged)));
            else
                x(notconverged) = x(notconverged) - m*polyval(p,x(notconverged))./polyval(dp,x(notconverged));
            end
        end
        
        % logical matrix with 1 if the entry has not converged and 0 if it
        % has
        t = abs(prev - x) > tol;
        
        % if an entry converges this iteration record how many iterations
        % it took
        conv(xor(notconverged, t)) = j;
        
        % update the logical matrix
        notconverged = t;
       
        % break out of the loop if every entry has converged
        if ~notconverged
            fprintf('converged after %g iterations \n', j)
            break
        end
           
        
         if dotime
            toc
         end
        
        prev = x;
    end

    s = conv;
    r = x;
end

